import random

from sqlalchemy import (Column, Integer, String, ForeignKey, Computed, Boolean)
from sqlalchemy.dialects.postgresql import JSONB, UUID, TSVECTOR
from sqlalchemy.orm import relationship

from .base_model import BaseModel


class SqlRequests:

    """ Хранилище SQL запросов """

    @staticmethod
    def transformation_to_ts_vector():
        """ запрос для генерации в ts_vector JSONB """

        return "to_tsvector('russian', jsonb_path_query_array(service_model_data,'strict $.**.visitor') ||" \
               " jsonb_path_query_array(service_model_data,'strict $.**.phone'))"


class ServiceRequests(BaseModel):

    """  Заявки """

    __tablename__ = 'service_requests'
    __table_args__ = {"extend_existing": True}

    request_number = Column(Integer, nullable=False, default=random.randint(1, 10_000))
    status = Column(String, default='created', nullable=False)
    user_uuid = Column(UUID)
    service_model_data = Column(JSONB)
    responsible = Column(String, default="resident")
    description = Column(String)
    is_manager = Column(Boolean, default=False)

    service_id = Column(Integer, ForeignKey('services.id'), nullable=False)
    apartment_id = Column(Integer, ForeignKey('apartments.id'), nullable=True)
    building_id = Column(Integer, ForeignKey('buildings.id'), nullable=True)

    service = relationship('Services', back_populates='service_requests', uselist=False)
    apartment = relationship('Apartments', uselist=False)
    public_space = relationship('Buildings', uselist=False)
    files = relationship('NextcloudFiles', back_populates='service_requests', uselist=True)
    orders = relationship('OrderModel', back_populates='service_requests', uselist=True)

    ts_vector = Column(TSVECTOR, Computed(SqlRequests.transformation_to_ts_vector(),  persisted=True))



from sqlalchemy import (Column, Integer, String, ForeignKey, Enum)
from sqlalchemy.orm import relationship

from .base_model import BaseModel


class NextcloudFiles(BaseModel):

    """  Хранение файлов в сервисе Nextcloud """

    __tablename__ = 'nextcloud_files'
    __table_args__ = {"extend_existing": True}

    file_name = Column(String, nullable=False)
    file_type = Column(String, default='New', nullable=False)
    file_path = Column(String, nullable=False)
    size = Column(Integer, nullable=True)
    access = Column(String, default='internal')
    expire = Column(Integer, default=24)

    service_requests_id = Column(Integer, ForeignKey('service_requests.id'))
    service_requests = relationship('ServiceRequests', back_populates='files', uselist=False)



import random

from sqlalchemy import (Column, Integer, String, ForeignKey)
from sqlalchemy.dialects.postgresql import JSONB, UUID
from sqlalchemy.orm import relationship

from .base_model import BaseModel


class OrderModel(BaseModel):

    """  Наряды """

    __tablename__ = 'orders'
    __table_args__ = {"extend_existing": True}

    user_uuid = Column(UUID)
    status = Column(String, default='created', nullable=False)
    description = Column(String)

    service_requests_id = Column(Integer, ForeignKey('service_requests.id'))
    service_requests = relationship('ServiceRequests', back_populates='orders', uselist=False)



from .services_guide_models import *
from .estate_models import *
from .house_management_models import *
from .base_model import Base
from .order_models import *
from .files_models import *
from .service_requests_models import *
